// Copyright (C) 2014 Paul S. Heidmann
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// For a copy of the GNU General Public License see
// <http://www.gnu.org/licenses/>.
//
// Author contact info:
//   Paul Heidmann
//   paul@heidmann.com

#include <fractalOne.hpp>

#include <stdexcept>

namespace FO=fractalOne;

namespace
{
FO::fractalParamsType::colorType colorForZero(
  const std::uint32_t maxIters,
  const FO::cmplxType& z,
  const std::uint32_t numIters )
{
  const FO::fltType scaleFactor{
    static_cast<FO::fltType>(numIters) / static_cast<FO::fltType>(maxIters) };
  const FO::fltType colorMagnitude{
    static_cast<FO::fltType>(0xff) * scaleFactor };
  const std::uint32_t color{ static_cast<std::uint32_t>(colorMagnitude) };

  if( std::abs( z + 6.0 ) < 0.1 )
    return( std::move( std::make_tuple(
			 color, 0x00, color ) ) );
  if( std::abs( z + 4.0 ) < 0.1 )
    return( std::move( std::make_tuple(
			 color, color, 0x00 ) ) );
  if( std::abs( z - 3.0 ) < 0.1 )
    return( std::move( std::make_tuple(
			 color, 0x00, 0x00 ) ) );
  if( std::abs( z - 5.0 ) < 0.1 )
    return( std::move( std::make_tuple(
			 0x00, color, 0x00 ) ) );
  if( std::abs( z - 7.0 ) < 0.1 )
    return( std::move( std::make_tuple(
			 0x00, 0x00, color ) ) );
  throw( std::runtime_error( "Unknown zero..." ) );
}

FO::cmplxType f( const FO::cmplxType& z )
{
  // zeros are -6, -4, 3, 5, 7
  const FO::cmplxType z_2{ z * z };
  const FO::cmplxType z_3{ z_2 * z };
  const FO::cmplxType z_4{ z_3 * z };
  const FO::cmplxType z_5{ z_4 * z };
  return( z_5 - (5.0 * z_4) - (55.0 * z_3) + (245.0 * z_2) + (654.0 * z) - 2520.0 );
}

FO::cmplxType f_prime( const FO::cmplxType& z )
{
  const FO::cmplxType z_2{ z * z };
  const FO::cmplxType z_3{ z_2 * z };
  const FO::cmplxType z_4{ z_3 * z };
  return( (5.0 * z_4) - (20.0 * z_3) - (165.0 * z_2) + (490.0 * z) + 654.0 );
}

}

std::shared_ptr<FO::fractalParamsType>
FO::getFractalParams( void )
{
#if 0
  const FO::cmplxType lowerLeft{ 4.5, 7.9 };
  const FO::cmplxType upperRight{ 5.7, 9.1 };
  const FO::cmplxType delta{ 0.0005, 0.0005 };
#else
  const FO::cmplxType lowerLeft{ 4.9, 8.4 };
  const FO::cmplxType upperRight{ 5.1, 8.6 };
  const FO::cmplxType delta{ 0.00005, 0.00005 };
#endif
  const FO::fltType zEpsilon{ 0.05 };
  const std::vector<FO::cmplxType> zeros{
    FO::cmplxType{ -6.0, 0.0 },
    FO::cmplxType{ -4.0, 0.0 },
    FO::cmplxType{ 3.0, 0.0 },
    FO::cmplxType{ 5.0, 0.0 },
    FO::cmplxType{ 7.0, 0.0 } };
  const std::uint32_t maxIters{ 50 };

  const std::shared_ptr<FO::fractalParamsType> fParams{
    std::make_shared<FO::fractalParamsType>(
      lowerLeft,
      upperRight,
      delta,
      zEpsilon,
      f,
      f_prime,
      zeros,
      maxIters,
      colorForZero ) };
  return( fParams );
}
