#pragma once

// Copyright (C) 2014 Paul S. Heidmann
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// For a copy of the GNU General Public License see
// <http://www.gnu.org/licenses/>.
//
// Author contact info:
//   Paul Heidmann
//   paul@heidmann.com

#include <memory>
#include <cstdint>
#include <complex>

#include <fractalParams.hpp>

/// \brief This namespace contains the equations and parameters that are
///        used to generate the second example fractal.
///
/// This namespace contains the parameters and equations needed to generate
/// the second example fractal. This fractal uses the function:\n
/// \f$f(z) = sin(z)\f$
/// \author Paul S. Heidmann
namespace fractalTwo
{
  typedef double fltType;
  typedef fractal::fractalParams<fltType> fractalParamsType;
  typedef std::complex<fltType> cmplxType;

  std::shared_ptr<fractalParamsType> getFractalParams( void );
}
