// Copyright (C) 2014 Paul S. Heidmann
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// For a copy of the GNU General Public License see
// <http://www.gnu.org/licenses/>.
//
// Author contact info:
//   Paul Heidmann
//   paul@heidmann.com

#include <tuple>
#include <memory>
#include <vector>
#include <cstdint>
#include <complex>
#include <iostream>
#include <stdexcept>
#include <pngFactory.hpp>
#include <newtonsMethod.hpp>
#include <fractalParams.hpp>
#include <threadedFractalEngine.hpp>

#define FRACTAL_TWO

#ifdef FRACTAL_ONE
#include <fractalOne.hpp>
#endif

#ifdef FRACTAL_TWO
#include <fractalTwo.hpp>
#endif

/// \brief This is the main (top level) method for the mkFractal program.
///
/// This program generates fractals, using Newton's method in the complex
/// plane. The colors indicate which zero of the generating function the
/// given point in the complex plane (as an initial guess for Newton's
/// method) converged to. The shading is derived from the number of
/// iterations required to reach the zero.
/// \author Paul S. Heidmann
int main( void )
{
  const std::shared_ptr<png::pngFactory> pngFact{
    std::make_shared<png::pngFactory>(
      2000,
      2000,
      "fractal.png" ) };

  const std::shared_ptr<fractal::fractalParams<double>> fParams{
#ifdef FRACTAL_ONE
    fractalOne::getFractalParams()
#endif
#ifdef FRACTAL_TWO
    fractalTwo::getFractalParams()
#endif
  };

  fractal::threadedFractalEngine<double> fEng{
    fParams,
    pngFact };

  fEng.doFractal();

  return( 0 );
}
